<?php
    /**
     * Created by JetBrains PhpStorm.
     * User: Jim
     * Date: 5/6/13
     * Time: 12:15 AM
     * To change this template use File | Settings | File Templates.
     */

    include_once 'CurlErrors.php';
    include_once 'CurlException.php';
    include_once 'InvalidArgException.php';

    class CurlFTP implements CurlErrors
    {
        private $baseURL;   // note this should be the BASE URL to your FTP server.  Try not to include users and passsords
                            // and file names.  ex)  ftp://myftp.com

        private $port = 21;
        private $user;
        private $password;
        private $setupOptions;
        private $lastResponse;
        private $bConnected;
        private $ch;
        private $lastCurlError;
        private $out;

        function __construct($baseURL)
        {
            $this->baseURL = $baseURL;
            $this->lastResponse = "";
            $this->bConnected = false;
            $this->ch = null;
            $this->lastCurlError = self::CURLE_OK;
            //$this->out = fopen("test.deb", "rw+");
        }

        private function setLastCurlError()
        {
            $this->lastCurlError = curl_errno($this->ch);
        }

        public function setUserID($user, $password)
        {
            $this->setUser($user);
            $this->setPassword($password);
        }

        public function setPort($port)
        {
            $this->port = $port;
        }

        public function getPort()
        {
            return $this->port;
        }

        public function setUser($user)
        {
            $this->user = $user;
        }

        public function getUser()
        {
            return $this->user;
        }

        public function setPassword($password)
        {
            $this->password = $password;
        }

        public function getPassword()
        {
            return $this->password;
        }

        public function getLastCurlError()
        {
            return $this->lastCurlError;
        }

        public function getBaseURL()
        {
            return $this->baseURL;
        }

        protected function setOptions()
        {
            //curl_setopt($this->ch, CURLOPT_CAPATH, dirname(__DIR__)."/CERTS");
            // reset all the values that were already set
            foreach ($this->setupOptions as $const => $value)
            {
                curl_setopt($this->ch, $const, $value);
            }
        }

        public function addSetupOpt($option, $value)
        {
            $this->setupOptions[$option] = $value;
            return $this;
        }

        public function isConnected()
        {
            return $this->bConnected;
        }

        public function getLastResponse()
        {
            return $this->lastResponse;
        }

        public function connect()
        {
            $this->ch = curl_init();
            $this->setOptions();
            curl_setopt($this->ch, CURLOPT_URL, $this->getBaseURL());
            $this->lastResponse = curl_exec($this->ch);
            $this->setLastCurlError();
            if ($this->getLastCurlError() == self::CURLE_OK)
            {
                $this->bConnected = true;
            } else
            {
                $this->bConnected = false;
            }
        }

        public function close()
        {
            curl_close($this->ch);
            //$this->setLastCurlError();
            $this->bConnected = false;
            $this->ch = null;
        }

        public function changeDir($dir)
        {
            if ($dir == "")
            {
                throw new InvalidArgException("NULL or zero length argument passed.");
            }

            curl_setopt($this->ch, CURLOPT_UPLOAD, false);
            curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, "cwd " . $dir);
            $this->lastResponse = curl_exec($this->ch);
            $this->setLastCurlError();
            //Clear out temp curl option.
            curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, null);
            curl_setopt($this->ch, CURLOPT_UPLOAD, null);

            if ($this->getLastCurlError() != self::CURLE_OK && $this->getLastCurlError() != self::CURLE_FTP_COULDNT_RETR_FILE)
            {
                throw new CurlException("cURL Exception Changing Directory [ $dir ] ", $this->getLastCurlError());
            }

            return $this->getLastCurlError();
        }

        public function getDir()
        {
            $cmd = "list";
            $ret = "";
            $f = fopen("denise.txt", "w+");

            curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, "pwd");
            curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($this->ch, CURLINFO_HEADER_OUT, true);
            //curl_setopt($this->ch, CURLOPT_WRITEFUNCTION, 'self::progress_function');
            curl_setopt($this->ch, CURLOPT_STDERR, $f);
            //curl_setopt($this->ch, CURLOPT_FILE, $this->out);
            //curl_setopt($this->ch, CURLOPT_FTPASCII, 1);

            //curl_setopt($this->ch, CURLOPT_QUOTE, $cmd);

            curl_setopt($this->ch, CURLOPT_UPLOAD, false);

            $this->lastResponse = curl_exec($this->ch);
            fflush($f);
            $cmd = fread($f, 2048);
            print "Read " . $cmd . "\n";

            $cmd = curl_getinfo($this->ch, CURLINFO_HTTP_CODE);

            print "Command " . $cmd . "\n";

            //$cmd = curl_getinfo($this->ch, CURLINFO_HTTP_);
            $cmd = curl_multi_getcontent($this->ch);

            print "Conntent " . $cmd . "\n";
            //$ret[] = curl_multi_info_read($this->ch);
            //print "Return " . $ret . "\n";
            $this->setLastCurlError();
            //Clear out temp curl option.
            curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, null);
            curl_setopt($this->ch, CURLOPT_UPLOAD, null);
            fclose($f);

            return $this->getLastResponse();
        }

        public function progress_function($ch, $str)
        {
            echo "Progress Function ----> " . $str;
            return strlen($str);
        }

    }
?>